/*----------------------------------------------------------------------- 
 * 
 * File Name: inspiralfrutils.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: inspiralfrutils.c,v 1.3 2003/06/17 07:16:22 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "inspiralfrutils.h"

/*
 *
 * functions to write data to frame files, since lal doesn't have this yet
 *
 */


FrameH *fr_add_proc_REAL4TimeSeries ( 
    FrameH                     *frame, 
    REAL4TimeSeries            *chan,
    const char                 *unit,
    const char                 *suffix
    )
{
  char          chname[256];
  struct        series fdata;

  LALSnprintf( chname, sizeof(chname), "%s_%s", chan->name, suffix );
    fdata.name = chname;
    fdata.tbeg = chan->epoch;
    memset( &fdata.tend, 0, sizeof(LIGOTimeGPS) );
    epoch_add( &fdata.tend, &(chan->epoch), 
        chan->deltaT * (REAL8) chan->data->length );
    fdata.dom = Time;
    fdata.type = FR_VECT_4R;
    fdata.step = (float) chan->deltaT;
    fdata.unit = unit;
    fdata.size = (size_t) chan->data->length;
    fdata.data = (float *) chan->data->data;
    return fr_add_proc_data( frame, &fdata );
}

FrameH *fr_add_proc_REAL4FrequencySeries ( 
    FrameH                     *frame, 
    REAL4FrequencySeries       *chan,
    const char                 *unit,
    const char                 *suffix
    )
{
  char          chname[256];
  struct        series fdata;

  LALSnprintf( chname, sizeof(chname), "%s_%s", chan->name, suffix );
    fdata.name = chname;
    fdata.tbeg = chan->epoch;
    memset( &fdata.tend, 0, sizeof(LIGOTimeGPS) );
    epoch_add( &fdata.tend, &chan->epoch, 
        (chan->data->length - 1) / (chan->deltaF * chan->data->length) );
    fdata.dom = Freq;
    fdata.type = FR_VECT_4R;
    fdata.step = (float) chan->deltaF;
    fdata.unit = unit;
    fdata.size = (size_t) chan->data->length;
    fdata.data = (float *) chan->data->data;
    return fr_add_proc_data( frame, &fdata );
}

FrameH *fr_add_proc_COMPLEX8FrequencySeries ( 
    FrameH                        *frame, 
    COMPLEX8FrequencySeries       *chan,
    const char                    *unit,
    const char                    *suffix
    )
{
  char          chname[256];
  struct        series fdata;

  LALSnprintf( chname, sizeof(chname), "%s_%s", chan->name, suffix );
    fdata.name = chname;
    fdata.tbeg = chan->epoch;
    memset( &fdata.tend, 0, sizeof(LIGOTimeGPS) );
    epoch_add( &fdata.tend, &chan->epoch, 
        (chan->data->length - 1) / (chan->deltaF * chan->data->length) );
    fdata.dom = Freq;
    fdata.type = FR_VECT_8C;
    fdata.step = (float) chan->deltaF;
    fdata.unit = unit;
    fdata.size = (size_t) chan->data->length;
    fdata.data = (float *) chan->data->data;
    return fr_add_proc_data( frame, &fdata );
}

FrameH *fr_add_proc_REAL8FrequencySeries ( 
    FrameH                     *frame, 
    REAL8FrequencySeries       *chan,
    const char                 *unit,
    const char                 *suffix
    )
{
  char          chname[256];
  struct        series fdata;
  char comment[] = "Generated by $Id: inspiralfrutils.c,v 1.3 2003/06/17 07:16:22 duncan Exp $";
  char seconds[] = "s";
  char hertz[]   = "Hz";
  struct FrVect     *vect;
  struct FrProcData *proc;
  size_t i;
  const char *channel = fdata.name;

  LALSnprintf( chname, sizeof(chname), "%s_%s", chan->name, suffix );
    fdata.name = chname;
    fdata.tbeg = chan->epoch;
    memset( &fdata.tend, 0, sizeof(LIGOTimeGPS) );
    epoch_add( &fdata.tend, &chan->epoch, 
        (chan->data->length - 1) / (chan->deltaF * chan->data->length) );
    fdata.dom = Freq;
    fdata.type = FR_VECT_8R;
    fdata.step = (float) chan->deltaF;
    fdata.unit = unit;
    fdata.size = (size_t) chan->data->length;

  if ( ! frame )
  {
    char src[2];
    src[0] = fdata.name[0];
    src[1] = 0;
    frame = FrameHNew( src );
    frame->run    = 1;
    frame->frame  = 1;
    frame->GTimeS = (int) fdata.tbeg.gpsSeconds;
    frame->GTimeN = (int) fdata.tbeg.gpsNanoSeconds;
    frame->dt     = epoch_diff( &fdata.tend, &fdata.tbeg );
  }

  vect = FrVectNew1D( channel, fdata.type, fdata.size, fdata.step,
      IS_TIME( fdata.dom) ? seconds : hertz, fdata.unit );
  proc = calloc( 1, sizeof( *proc ) );
  proc->classe     = FrProcDataDef();
#if defined FR_VERS && FR_VERS < 5000
  proc->sampleRate = IS_TIME( fdata.dom) ? 1.0 / fdata.step : -1;
#endif
  proc->fShift     = 0;
  proc->data       = vect;
  proc->next       = frame->procData;
  frame->procData  = proc;
  FrStrCpy( &proc->name, channel );
  FrStrCpy( &proc->comment, comment );
  for ( i = 0; i < fdata.size; ++i )
  {
    vect->dataF[i] = fdata.data[i];
  }

  return frame;
}
